from .transport import send


def refund_transaction(cid, property_id: str, transaction_id: str):
    """Create a Google Analytics transaction refund using the Measurement Protocol API.
    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param transaction_id: Transaction ID
    :return: HTTP response status
    """

    payload = {
        'cid': cid,
        'ti': transaction_id,
        't': 'event',
        'ec': 'Ecommerce',
        'ea': 'Refund transaction',
        'ni': 1,
        'pa': 'refund'
    }
    send(payload, property_id)


def refund_items(cid, property_id: str, transaction_id: str, item_code: str, item_quantity: int):
    """Create a Google Analytics item refund using the Measurement Protocol API.

    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param transaction_id: Transaction ID
    :param item_code: Item code, i.e. ABC1
    :param item_quantity: Quantity of units to refund, i.e. 3
    :return: HTTP response status
    """

    payload = {
        'cid': cid,
        'ti': transaction_id,
        'pr1id': item_code,
        'pr1qt': item_quantity,
        't': 'event',
        'ec': 'Ecommerce',
        'ea': 'Refund transaction',
        'ni': 1,
        'pa': 'refund'
    }
    send(payload, property_id)


def promotion_impression(cid,
                         property_id: str,
                         document_hostname: str,
                         document_path: str,
                         document_title: str,
                         promotion_index: int,
                         promotion_id: str,
                         promotion_name: str,
                         promotion_creative: str,
                         promotion_position: str):
    """Create a Google Analytics promotion impression using the Measurement Protocol API.

    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param document_hostname: Document hostname, i.e. example.com
    :param document_path: Document path, i.e. /about
    :param document_title: Document title, i.e. About us
    :param promotion_index: Promotion index slot (integer between 1 and 200)
    :param promotion_id: Promotion ID, i.e. PROMO_123
    :param promotion_name: Promotion name, i.e. Summer sale
    :param promotion_creative: Promotion creative, i.e. Summer sale banner
    :param promotion_position: Promotion position, i.e. Top left slot
    :return: HTTP response status
    """

    payload = {
        'cid': cid,
        't': 'pageview',
        'dh': document_hostname,
        'dp': document_path,
        'dt': document_title,
        'promo' + str(promotion_index) + 'id': promotion_id,
        'promo' + str(promotion_index) + 'nm': promotion_name,
        'promo' + str(promotion_index) + 'cr': promotion_creative,
        'promo' + str(promotion_index) + 'ps': promotion_position,
    }
    send(payload, property_id)


def promotion_click(cid,
                    property_id: str,
                    promotion_index: int,
                    promotion_id: str,
                    promotion_name: str,
                    promotion_creative: str,
                    promotion_position: str):
    """Create a Google Analytics promotion click using the Measurement Protocol API.

    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param promotion_index: Promotion index slot (integer between 1 and 200)
    :param promotion_id: Promotion ID, i.e. PROMO_123
    :param promotion_name: Promotion name, i.e. Summer sale
    :param promotion_creative: Promotion creative, i.e. Summer sale banner
    :param promotion_position: Promotion position, i.e. Top left slot
    :return: HTTP response status
    """

    payload = {
        'cid': cid,
        't': 'event',
        'ec': 'Internal promotions',
        'ea': 'Click',
        'el': promotion_name,
        'promoa': 'click',
        'promo' + str(promotion_index) + 'id': promotion_id,
        'promo' + str(promotion_index) + 'nm': promotion_name,
        'promo' + str(promotion_index) + 'cr': promotion_creative,
        'promo' + str(promotion_index) + 'ps': promotion_position,
    }
    send(payload, property_id)


def checkout_option(cid, property_id: str, transaction_id: str, checkout_step_number: int, checkout_option_label: str):
    """Record a Google Analytics checkout option using the Measurement Protocol API.

    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param transaction_id: Transaction ID
    :param checkout_step_number: Checkout step number, i.e. 2
    :param checkout_option_label: Checkout option label, i.e. FedEx
    :return: HTTP response status
    """
    payload = {
        'cid': cid,
        'ti': transaction_id,
        't': 'event',
        'ec': 'Checkout',
        'ea': 'Option',
        'pa': 'checkout_option',
        'cos': checkout_step_number,
        'col': checkout_option_label,
    }
    send(payload, property_id)


def checkout_step(cid,
                  property_id: str,
                  transaction_id: str,
                  document_hostname: str,
                  document_path: str,
                  document_title: str,
                  checkout_step_number: int,
                  checkout_step_label: str):
    """Record a Google Analytics checkout step using the Measurement Protocol API.

    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param transaction_id: Transaction ID
    :param document_hostname: Document hostname, i.e. example.com
    :param document_path: Document path, i.e. /payment
    :param document_title: Document title, i.e. Checkout complete
    :param checkout_step_number: Checkout step number, i.e. 5
    :param checkout_step_label: Checkout step label, i.e. Checkout complete
    :return:
    """
    payload = {
        'cid': cid,
        'ti': transaction_id,
        't': 'pageview',
        'dh': document_hostname,
        'dp': document_path,
        'dt': document_title,
        'ec': 'Checkout',
        'ea': 'Option',
        'pa': 'checkout',
        'cos': checkout_step_number,
        'col': checkout_step_label,
    }
    send(payload, property_id)


def enhanced_transaction(cid,
                         property_id: str,
                         document_hostname: str,
                         document_path: str,
                         document_title: str,
                         transaction_id: str,
                         transaction_revenue: float,
                         transaction_tax: float,
                         transaction_shipping: float,
                         transaction_coupon: str,
                         items: dict):
    """Record a Google Analytics enhanced ecommerce transaction step using the Measurement Protocol API.
    This function takes core parameters for a transaction and a nested Python dictionary of items and then
    formats and merges the payloads before passing them to the API.

    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param document_hostname: Document hostname, i.e. example.com
    :param document_path: Document path, i.e. /payment
    :param document_title: Document title, i.e. Checkout complete
    :param transaction_id: Transaction ID, i.e. 12345
    :param transaction_revenue: Transaction revenue, i.e. 199.99
    :param transaction_tax: Transaction tax, i.e. 49.99
    :param transaction_shipping: Transaction shipping, i.e. 0
    :param transaction_coupon: Transaction coupon code, i.e. WELCOME
    :param items: Python dictionary of item level data
    :return: HTTP Status
    """
    payload = {
        'cid': cid,
        't': 'pageview',
        'pa': 'purchase',
        'dh': document_hostname,
        'dp': document_path,
        'dt': document_title,
        'ti': transaction_id,
        'tr': transaction_revenue,
        'tt': transaction_tax,
        'ts': transaction_shipping,
        'tcc': transaction_coupon,
    }

    payload = {**payload, **enhanced_transaction_items(items)}
    send(payload, property_id)


def enhanced_transaction_items(items: dict):
    """Return a formatted payload of items for a Google Analytics enhanced ecommerce transaction.

    :param items: Nested Python dictionary containing item data.
    :return: Formatted payload of items matching the below format.
    :example:

    items = {
    1: {'product_id': 'ABC1',
        'product_name': 'Product 1',
        'product_price': 19.99,
        'product_quantity': 1,
        'product_brand': 'Apple',
        'product_category': 'Phones',
        'product_variant': '256GB',
        },
    2: {'product_id': 'ABC2',
        'product_name': 'Product 2',
        'product_price': 20.99,
        'product_quantity': 2,
        'product_brand': 'Apple',
        'product_category': 'Phones',
        'product_variant': '500GB',
        },
    }
    """

    items_payload = {}

    for item in items:
        prefix = 'pr{0}'.format(item)
        payload_item = {}

        if 'product_id' in items[item]:
            payload_item[prefix + 'id'] = items[item]['product_id']

        if 'product_name' in items[item]:
            payload_item[prefix + 'nm'] = items[item]['product_name']

        if 'product_price' in items[item]:
            payload_item[prefix + 'pr'] = items[item]['product_price']

        if 'product_quantity' in items[item]:
            payload_item[prefix + 'qt'] = items[item]['product_quantity']

        if 'product_brand' in items[item]:
            payload_item[prefix + 'br'] = items[item]['product_brand']

        if 'product_category' in items[item]:
            payload_item[prefix + 'ca'] = items[item]['product_category']

        if 'product_variant' in items[item]:
            payload_item[prefix + 'va'] = items[item]['product_variant']

        items_payload = {**items_payload, **payload_item}

    return items_payload



from .transport import send


def custom_dimension(cid, property_id: str, index: str, value: str):
    """Populate a Google Analytics custom dimension using the Measurement Protocol API.
    Google Analytics only provides 20 custom dimension slots so you will need to use them wisely.

    :param cid: Client ID
    :param index: The custom dimension index you wish to populate (only 20 are available).
    :param value: The value you wish to assign to the custom dimension
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :return: HTTP response status
    """

    payload = {
        'cid': cid,
        't': 'event',
        'ec': 'Measurement Protocol requests',
        'ea': 'cd{0}'.format(index) + '=' + value,
        'ni': 1,
        'cd{0}'.format(index): value,
    }
    send(payload, property_id)


def custom_metric(cid, property_id: str, index: str, value: str):
    """Populate a Google Analytics custom metric using the Measurement Protocol API.
    Google Analytics only provides 20 custom metric slots so you will need to use them wisely.

    :param cid: Client ID
    :param index: The custom metric index you wish to populate (only 20 are available).
    :param value: The value you wish to assign to the custom metric
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :return: HTTP response status
    """

    payload = {
        'cid': cid,
        't': 'event',
        'ec': 'Measurement Protocol requests',
        'ea': 'cm{0}'.format(index) + '=' + value,
        'ni': 1,
        'cm{0}'.format(index): value,
    }
    send(payload, property_id)



import requests
import random

endpoint = 'https://www.google-analytics.com/collect'
user_agent = 'pygamp'


def send(payload, property_id):
    """Send a payload to Google Analytics using the Measurement Protocol API.

    :param payload: Python dictionary of URL key value pairs
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :return: HTTP response status
    """

    required_payload = {
        'v': 1,
        'tid': property_id,
        'aip': 1,
        'z': random.random()
    }

    final_payload = {**required_payload, **payload}
    response = requests.post(url=endpoint,
                             data=final_payload,
                             headers={'User-Agent': user_agent},
                             timeout=5.0)
    print(response.request.body)
    return response




from .transport import send


def screenview(cid,
               property_id: str,
               app_name: str,
               app_version: str,
               screen_name: str,
               app_id: str = None,
               app_installer_id: str = None,
               ):
    """Create a Google Analytics app screenview using the Measurement Protocol API.

    :param app_name: Application name
    :param app_installer_id: Application installer ID
    :param app_id: Application ID
    :param screen_name: Screen name, i.e. Home
    :param app_version: Application version number, i.e. 1.0.3
    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :return: HTTP response status
    """

    payload = {
        'cid': cid,
        't': 'screenview',
        'an': app_name,
        'av': app_version,
        'aid': app_id,
        'aiid': app_installer_id,
        'cd': screen_name
    }
    send(payload, property_id)



from .transport import send


def pageview(cid,
             property_id: str,
             document_path: str = None,
             document_hostname: str = None,
             document_title: str = None):
    """Create a Google Analytics pageview using the Measurement Protocol API.

    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param document_path: Document path, i.e. /stuff
    :param document_hostname: Document hostname, i.e. example.com
    :param document_title: Document title, i.e. About us
    :return: HTTP response status
    """

    payload = {
        'cid': cid,
        't': 'pageview',
        'dp': document_path,
        'dh': document_hostname,
        'dt': document_title
    }
    send(payload, property_id)



from .transport import send


def exception(cid,
              property_id: str,
              exception_description: str,
              exception_fatal: int = 1):
    """Create a Google Analytics exception using the Measurement Protocol API.

    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param exception_description: Description of exception, i.e. 'Transaction not found'
    :param exception_fatal: Pass a 1 value for fatal exceptions or a zero for non-fatal
    :return: HTTP response status
    """

    payload = {
        'cid': cid,
        't': 'exception',
        'exd': exception_description,
        'exf': exception_fatal,
    }
    send(payload, property_id)



from .transport import send


def event(cid,
          property_id: str,
          category: str,
          action: str,
          label: str = None,
          value: int = None,
          non_interactive: int = 0):
    """Create a Google Analytics event using the Measurement Protocol API.

    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param category: Event category string
    :param action: Event action string
    :param label: Optional event label string
    :param value: Optional event value integer
    :param non_interactive: Specify whether the hit type is non-interactive by passing 1
    :return: HTTP response status
    """

    payload = {
        'cid': cid,
        't': 'event',
        'ec': category,
        'ea': action,
        'el': label,
        'ev': str(int(value)),
        'ni': non_interactive
    }
    send(payload, property_id)



from .transport import send


def social(cid,
           property_id: str,
           social_action: str,
           social_network: str,
           social_target: str):
    """Create a Google Analytics social interaction using the Measurement Protocol API.

    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param social_action: Social hit type, i.e. like
    :param social_network: Social network, i.e. facebook
    :param social_target: Social target or document name, i.e. /about-us
    :return: HTTP response status
    """

    payload = {
        'cid': cid,
        't': 'social',
        'social_action': social_action,
        'social_network': social_network,
        'social_target': social_target
    }
    send(payload, property_id)



from .app import screenview
from .custom_definitions import custom_metric, custom_dimension
from .ecommerce import item, transaction
from .enhanced_ecommerce import refund_items, refund_transaction, enhanced_transaction, enhanced_transaction_items
from .enhanced_ecommerce import promotion_impression, promotion_click
from .enhanced_ecommerce import checkout_step, checkout_option
from .events import event
from .exceptions import exception
from .pageviews import pageview
from .social_interactions import social
from .transport import send



from .transport import send


def transaction(cid,
                property_id: str,
                transaction_id: str,
                transaction_revenue: float,
                transaction_tax: float,
                transaction_shipping: float,
                transaction_currency: str,
                transaction_affiliation: str = None,
                transaction_coupon: str = None,
                ):
    """Create a Google Analytics transaction using the Measurement Protocol API.
    This should be used in conjunction with individual item hits containing each SKU,
    and Google Analytics will link them together using the common transaction_id.

    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param transaction_coupon: Transaction coupon used
    :param transaction_shipping: Transaction shipping charge
    :param transaction_tax: Transaction tax charge
    :param transaction_revenue: Transaction revenue
    :param transaction_id: Transaction ID
    :param transaction_affiliation: Affiliation, i.e. Website
    :param transaction_currency: Transaction currency code, i.e. GBP
    :return: HTTP response status
    """

    payload = {'cid': cid,
               't': 'transaction',
               'ti': transaction_id,
               'ta': transaction_affiliation,
               'tr': transaction_revenue,
               'tt': transaction_tax,
               'ts': transaction_shipping,
               'tcc': transaction_coupon,
               'cu': transaction_currency,
               }
    send(payload, property_id)


def item(cid,
         property_id: str,
         transaction_id: str,
         item_name: str,
         item_price: float,
         item_quantity: int,
         item_code: str,
         item_currency: str,
         item_variation: str = None,
         ):
    """Create a Google Analytics transaction using the Measurement Protocol API.
    This should be used in conjunction with individual item hits containing each SKU,
    and Google Analytics will link them together using the common transaction_id.

    :param cid: Client ID
    :param property_id: Universal Analytics property ID, i.e. UA-123456-1
    :param transaction_id: Transaction ID
    :param item_variation: Item variation or category, i.e. Widgets
    :param item_currency: Item currency code, i.e. GBP
    :param item_code: Item code, i.e. ABC1234
    :param item_quantity: Item quantity, i.e. 5
    :param item_price: Item unit price, i.e. 1.99
    :param item_name: Item name, i.e. Purple Widget
    :return: HTTP response status
    """

    payload = {'cid': cid,
               't': 'transaction',
               'ti': transaction_id,
               'in': item_name,
               'ip': item_price,
               'iq': item_quantity,
               'ic': item_code,
               'iv': item_variation,
               'cu': item_currency,
               }
    send(payload, property_id)
